# -*- coding: utf-8 -*-

# Le but de ce programme est de visualiser la conformalité des applications analytiques complexes.


import numpy as np
import matplotlib.pyplot as plt


# Tout d'abord, on définit quelques fonctions : exponentielle et (1+z**2)**(-1).
# N'hésitez pas à implémenter les fonctions de votre choix !

def exponentielle_complexe (XY) :
    return (np.exp(XY[0]) * np.cos(XY[1]), np.exp(XY[0]) * np.sin(XY[1]))

def inverse_complexe (XY) :
    return (XY[0]/(XY[0]**2+XY[1]**2), -XY[1]/(XY[0]**2+XY[1]**2))

def rationnelle_complexe (XY) :
    a = 1+XY[0]**2-XY[1]**2
    b = 2*XY[0]*XY[1]
    return (a/(a**2+b**2), -b/(a**2+b**2))


# Ensuite, définissons des courbes.

def horizontale (y) :
    return [(k/100.0, y) for k in range(-200, 200)]

def verticale (x) :
    return [(x, k/100.0) for k in range(-200, 200)]

def arc_cercle () :
    return [(1 + np.cos(k/100.0), np.sin(k/100.0)) for k in range(0, 314)]


# On peut afficher certaines de ces courbes.
# Attention : le but est de mesurer des angles. Il faut prendre la même échelle en abscisse et en ordonnée (repère orthornormé).

def graphique_courbe () :
    horizontale_abs, horizontale_ord = zip(*horizontale(0.5))
    verticale_abs, verticale_ord = zip(*verticale(0.5))
    arc_abs, arc_ord = zip(*arc_cercle())
    plt.clf() # Nettoie la mémoire de matplotlib de ce qui pourrait traîner.
    plt.gca().set_aspect('equal') # Même échelle en abscisse et en ordonnée.
    plt.plot(horizontale_abs, horizontale_ord, c='blue')
    plt.plot(verticale_abs, verticale_ord, c='red')
    plt.plot(arc_abs, arc_ord, c='green')
    plt.show() # Affiche le graphique.

# Et maintenant, on affiche les images de ces courbes par une fonction complexe 'fonc'.

def graphique_image_courbe (fonc) :
    horizontale_abs, horizontale_ord = zip(*map(fonc, horizontale(0.5)))
    verticale_abs, verticale_ord = zip(*map(fonc, verticale(0.5)))
    arc_abs, arc_ord = zip(*map(fonc, arc_cercle()))
    plt.clf() # Nettoie la mémoire de matplotlib de ce qui pourrait traîner.
    plt.gca().set_aspect('equal') # Même échelle en abscisse et en ordonnée.
    plt.plot(horizontale_abs, horizontale_ord, c='blue')
    plt.plot(verticale_abs, verticale_ord, c='red')
    plt.plot(arc_abs, arc_ord, c='green')
    plt.show() # Affiche le graphique.

# graphique_courbe()
# graphique_image_courbe(exponentielle_complexe)
# graphique_image_courbe(rationnelle_complexe)
    
# La conservation des angles se voit mieux en affichant l'image d'un quadrillage.
    
def graphique_quadrillage () :
    plt.clf() # Nettoie la mémoire de matplotlib de ce qui pourrait traîner.
    plt.gca().set_aspect('equal') # Même échelle en abscisse et en ordonnée.
    for k in range(-7, 8) :
        horizontale_abs, horizontale_ord = zip(*horizontale(k*0.25))
        plt.plot(horizontale_abs, horizontale_ord, c='blue')
    for k in range(-7, 8) : # Nouvelle boucle, pour que les verticales soient affichées après les horizontales.
        verticale_abs, verticale_ord = zip(*verticale(k*0.25))
        plt.plot(verticale_abs, verticale_ord, c='red')
    plt.show() # Affiche le graphique.

def graphique_image_quadrillage_exponentielle () :
    plt.clf() # Nettoie la mémoire de matplotlib de ce qui pourrait traîner.
    plt.gca().set_aspect('equal') # Même échelle en abscisse et en ordonnée.
    for k in range(-7, 8) :
        horizontale_abs, horizontale_ord = zip(*map(exponentielle_complexe, horizontale(k*0.25)))
        plt.plot(horizontale_abs, horizontale_ord, c='blue')
    for k in range(-7, 8) : # Nouvelle boucle, pour que les images de verticales soient affichées après les images d'horizontales.
        verticale_abs, verticale_ord = zip(*map(exponentielle_complexe, verticale(k*0.25)))
        plt.plot(verticale_abs, verticale_ord, c='red')
    plt.show() # Affiche le graphique.

def graphique_image_quadrillage_inverse () :
    plt.clf() # Nettoie la mémoire de matplotlib de ce qui pourrait traîner.
    plt.gca().set_aspect('equal') # Même échelle en abscisse et en ordonnée.
    for k in range(-7, 8) :
        if k != 0 :
            horizontale_abs, horizontale_ord = zip(*map(inverse_complexe, horizontale(k*0.25)))
            plt.plot(horizontale_abs, horizontale_ord, c='blue')
    for k in range(-7, 8) : # Nouvelle boucle, pour que les images de verticales soient affichées après les images d'horizontales.
        if k != 0 :
            verticale_abs, verticale_ord = zip(*map(inverse_complexe, verticale(k*0.25)))
            plt.plot(verticale_abs, verticale_ord, c='red')
    plt.show() # Affiche le graphique.

def graphique_image_quadrillage_rationnelle () :
    plt.clf() # Nettoie la mémoire de matplotlib de ce qui pourrait traîner.
    plt.gca().set_aspect('equal') # Même échelle en abscisse et en ordonnée.
    for k in range(-7, 8) :
        if k != -4 and k != 4 :
            horizontale_abs, horizontale_ord = zip(*map(rationnelle_complexe, horizontale(k*0.25)))
            plt.plot(horizontale_abs, horizontale_ord, c='blue')
    for k in range(-7, 8) : # Nouvelle boucle, pour que les images de verticales soient affichées après les images d'horizontales.
        if k != 0 :
            verticale_abs, verticale_ord = zip(*map(rationnelle_complexe, verticale(k*0.25)))
            plt.plot(verticale_abs, verticale_ord, c='red')
    plt.show() # Affiche le graphique.

# graphique_quadrillage()
# graphique_image_quadrillage_exponentielle()
# graphique_image_quadrillage_inverse()
graphique_image_quadrillage_rationnelle()